// $Id: accountlist.cpp,v 1.7 2004/02/10 02:32:11 martin Exp $

// accountlist.cpp, (C) 2004 Martin Henne

#include "io.h"
#include "accountlist.h"
#include "timespan.h"
#include "activity.h"
#include "activityaccount.h"
#include <cstdio>
#include <cstdlib>
#include <qstring.h>
#include <qsortedlist.h>



AccountList::AccountList(QSortedList<Activity>& allActivities)
    : _index(-1) {
    _accounts.setAutoDelete(true);
    create(allActivities);
}



void AccountList::create(QSortedList<Activity>& allActivities) {
    //std::fprintf(stderr,"AccountList::create() called\n");
    clear();
    Activity* activity = allActivities.first();
    while(activity) {
	this->addActivity(*activity);
	activity = allActivities.next();
    }
    /*
    std::fprintf(
	stderr,
	"AccountList::create(): created list of size `%d'\n",
	_accounts.count()
	);
	*/
}



ActivityAccount* AccountList::getNext() {
    //std::fprintf(stderr,"AccountList::getNext(): called\n");
    if(!hasNext()) // enough accounts stored?
	return 0;
    ++_index; // index counter
    if(_index == 0) {// first element?
	return _accounts.first();
    }
    // regular element, not first, but valid
    return _accounts.next();
}



bool AccountList::hasNext() const {
    //std::fprintf(stderr,"AccountList::hasNext() called\n");
    if(_index < (size()-1) ) {
	return true;
    }
    return false;
}



void AccountList::rewind() const {
    _index = -1;
    //ActivityAccount* getFirst = _accounts.first();
}



void AccountList::addActivity(const Activity& activity) {
    if( !exists(activity.getName()) ) {
	_accounts.append(new ActivityAccount(activity) );
	return;
    }
    
    ActivityAccount* a = getByName(activity.getName());
    if(a) {
	a->addTimeSpan(activity.getTimeSpan());
	return;
    }
    
    // impossible to get here
    println()("AccountList::addActivity(): fatal error");
}



void AccountList::clearAccount(const QString& name) {
    ActivityAccount* a = _accounts.first();
    while(a != NULL) {
	if(a->getName() == name) {
	    _accounts.remove(a);
	    return;
	}
	a = _accounts.next();
    }
}



void AccountList::addAccount(const ActivityAccount& account) {
    if(!exists(account.getName()))
	_accounts.append(new ActivityAccount(account));
    
    // create error message
    QString s("AccountList::addAccount(): '");
    s += account.getName() + "' allready exists";
    println()(s);
}



void AccountList::changeAccount(const ActivityAccount&) {
    println()("AccountList::changeAccount(account): deprecated call");
    std::exit(1);
}



void AccountList::changeAccount(const QString&, const TimeSpan&) {
    println()("AccountList::changeAccount(name,span): deprecated call");
    std::exit(1);
}



void AccountList::clear() {
    _accounts.clear();
    _index = -1;
}



bool AccountList::exists(const QString& name) {
    ActivityAccount* a = _accounts.first();
    while(a != NULL) {
	if(a->getName() == name)
	    return true;
	a = _accounts.next();
    }
    return false;
}



bool AccountList::exists(const ActivityAccount& account) {
    return exists(account.getName());
}



int AccountList::size() const {
    return _accounts.count();
}



ActivityAccount* AccountList::getByName(const QString& name) {
    //std::fprintf(stderr,"AccountList::getByName() called \n");
    ActivityAccount* a = _accounts.first();
    int i=0;
    while(a) {
	if(a->getName() == name) {
	    //std::fprintf(stderr,"AccountList::getByName(): returning '%s'\n",name.latin1() );
	    return a;
	}
	a = _accounts.next();
	++i;
    }
    return 0;
}



void AccountList::replace(const ActivityAccount& account) {
    clearAccount(account.getName());
    addAccount(account);
}



void AccountList::setListBoxIndex(const QString& name, int index) {
    ActivityAccount* a = _accounts.first();
    while(a != NULL) {
	if(a->getName() == name) {
	    a->setListBoxIndex(index);
	    return;
	}
	a = _accounts.next();
    }
    println()(QString
	      ("AccountList::setListBoxIndex(): can't set index for '"+name+"'")
	      );
}



void AccountList::changeAccount(int, const TimeSpan&) {
    println()
	(QString("AccountList::changeAccount(): no longer supported"));
    std::exit(1);
}



ActivityAccount* AccountList::getByListBoxIndex(int index) {
    ActivityAccount* a = _accounts.first();
    while(a != NULL) {
	if(a->getListBoxIndex() == index) {
	    return a;
	}
	a = _accounts.next();
    }
    QString s;
    s.sprintf("AccountList::getByListBoxIndex(): no such index: %d",index);
    println()(s);
    
    return 0;
}



TimeSpan AccountList::getOverallSum() {
    ActivityAccount* a = _accounts.first();
    TimeSpan span;
    while(a != NULL) {
	span += a->getTimeSpan();
	a = _accounts.next();
    }
    return span;
}



TimeSpan AccountList::getAccountSum(const QString& name) {
    ActivityAccount* a = getByName(name);
    if(!a) {
	QString s;
	s.sprintf("AccountList::getAccountSum(): '%s' not found",name.latin1());
	println()(s);
	TimeSpan t;
	return t;
    }
    return a->getTimeSpan();
}



TimeSpan AccountList::getOverallSum(const Activity& currentActivity) {
    return ( currentActivity.getTimeSpan() + getOverallSum() );
}



bool AccountList::empty() const {
    return (_accounts.count() == 0);
}




