// $Id: activity.cpp,v 1.11 2004/02/10 02:32:11 martin Exp $

// activity.cpp, (C) 2003 Martin Henne

//#include <iostream>
#include <qdatetime.h>
#include <qstring.h>
#include <qxml.h>
#include "activity.h"
#include "timediff.h"
#include "stringparser.h"
#include "io.h"
#include "timespan.h"

Activity::Activity() {
    _name = "";
    _isCurrent = false;
    _index = -1;
}


Activity::Activity(const QString& name) {
    _name = name;
    _start = QDateTime::currentDateTime();
    _end = _start;
    _isCurrent = false;
    _index = -1;
}


Activity::Activity(
    const QString& name,
    const QDateTime& start
    ) : _start(start), _name(name) {
    _end = _start;
    _isCurrent = false;
    _index = -1;
}


Activity::Activity(const QXmlAttributes& attr) {
    _name = attr.value("name");
    QString activityStartString = attr.value("start");
    _start = StringParser::getDateTime(activityStartString);
    QString activityEndString = attr.value("end");
    _end = StringParser::getDateTime(activityEndString);
    _isCurrent = false;
    _index = -1;
    //println()(QString("Activity(QXmlAttributes): ")+toString());
}


Activity::Activity(const Activity& other) {
    this->_start = other._start;
    this->_end = other._end;
    this->_name = other._name;
    this->_minutes = other._minutes;
    this->_isCurrent = other._isCurrent;
    this->_index = other._index;
}


void Activity::setEnd(const QDateTime& arg) {
    _end = arg;
}


QString Activity::toXmlString(const QString elementName) const {
    QString indent = "\t";
    QString s(indent+"<"+elementName+" name=\"");
    s += _name + "\"";
    s += " start=\"" + StringParser::toString(_start) + "\"";
    s += " end=\"" + StringParser::toString(_end) + "\"/>";
    return s;
}


QString Activity::toString() const {
    //std::cerr << __FILE__ << " toString()" << std::endl;
    QString s;
    s.sprintf(
	"\"%s\" (%s) - %s",
	_name.latin1(),
	getTimeSpan().toShortString().latin1(),
	_start.toString().latin1()
	);
    return s;
}


TimeSpan Activity::getDuration() const {
    int secs = _start.secsTo(_end);
    return TimeSpan(secs);
}



TimeSpan Activity::getTimeSpan() const {
    return getDuration();
}



bool Activity::operator<(const Activity& other) const {
    if ( this->_name < other._name ) {
	return true;
    }
    if ( this->_name > other._name ) {
	return false;
    }
    if ( this->_start < other._start ) {
	return true;
    }
    if ( this->_start > other._start ) {
	return false;
    }
    if ( this->_end < other._end ) {
	return true;
    }
    return false;
}


bool Activity::operator==(const Activity& other) const {
    if(
	other._name == this->_name &&
		     other._start == this->_start &&
				   other._end == this->_end
					       )
	return true;
    
    return false;
}


bool Activity::operator>(const Activity& other) const {
    if(!operator<(other) && !operator==(other))
	return true;
    
    return false;
}


bool Activity::empty() const {
    return (getName() == "");
}


void Activity::clear() {
    _name = "";
}


void Activity::setName(const QString& name) {
    _name = name;
}


void Activity::setCurrent(bool isCurrent) {
    _isCurrent = isCurrent;
}


bool Activity::getCurrent() const {
    return isCurrent();
}


bool Activity::isCurrent() const {
    return _isCurrent;
}

bool Activity::current() const {
    return _isCurrent;
}

void Activity::current(bool arg) {
    setCurrent(arg);
}


QString Activity::toDateTimeString(const QDateTime& dt) const {
    QString s;
    QDate d = dt.date();
    QTime t = dt.time();
    s.sprintf(
	"%s, %d. %s %.4d, %.2d:%.2d",
	d.dayName( d.dayOfWeek() ).latin1(),
	d.day(),
	d.monthName( d.month() ).latin1(),
	d.year(),
	t.hour(),
	t.minute());
    return s;
}


