// $Id: activity.h,v 1.12 2004/02/10 02:32:11 martin Exp $

// activity.h, (C) 2003 Martin Henne

/*
	This class represents one activity whith one start- and one end-time and
	a name. Don't get confused about the entries in the listBox that lists
	'Activities'. It does actually list the some of all passed time spent
	on activities that have the same name!
*/

#ifndef _activity_h
#define _activity_h

#include <qstring.h>
#include <qdatetime.h>
#include "timespan.h"

class QXmlAttributes;

class Activity {
public:
	Activity();
	explicit Activity(const QString& name);
	explicit Activity(const QXmlAttributes&);
	Activity(const QString& name,const QDateTime& start);
	Activity(const Activity& other);
	~Activity() {}

	void setStart(const QDateTime& arg) { _start = arg; }
	void setEnd(const QDateTime& arg);
	QDateTime getStart() const { return _start; }
	QDateTime getEnd() const { return _end; }

	QString getName() const { return _name; }
	void setName(const QString& name);
	QString toXmlString(const QString elementName = "activity") const;
	QString toString() const;

	QString getStartString() const { return toDateTimeString(this->getStart()); }
	QString getEndString() const { return toDateTimeString(this->getEnd()); }
	QString getTimeSpanString() const { return getTimeSpan().toShortString(); }
	
	TimeSpan getDuration() const;
	TimeSpan getTimeSpan() const;

	bool empty() const;
	void clear();

	bool operator<(const Activity& other) const;
	bool operator>(const Activity& other) const;
	bool operator==(const Activity& other) const;
	bool isCurrent() const;
	bool getCurrent() const;
	void setCurrent(bool);
	void current(bool arg);
	bool current() const;
	void setIndex(int i) { _index = i; }
	int getIndex() const { return _index; }


private:
	// take care of copy construcor!
	QDateTime _start;
	QDateTime _end;
	QString _name;
	float _minutes;
	bool _isCurrent;
	int _index;
	QString toDateTimeString(const QDateTime& dt) const;
};


#endif
