// $Id: activityaccount.cpp,v 1.5 2004/02/07 17:22:05 martin Exp $

// activityaccount.cpp, (C) 2004 Martin Henne

#include "activityaccount.h"
#include "activity.h"
#include "timespan.h"
#include <qstring.h>

ActivityAccount::ActivityAccount() {
	_name = "";
	_balance = TimeSpan(0,0,0);
	_listBoxIndex = -1;
	_report = true;
}

ActivityAccount::ActivityAccount(const QString& arg) : _name(arg) {
	_balance = TimeSpan(0,0,0);
	_listBoxIndex = -1;
	_report = true;
}


ActivityAccount::ActivityAccount(const QString& arg, const TimeSpan& span)
: _name(arg), _balance(span), _listBoxIndex(-1), _report(true) {
}


ActivityAccount::ActivityAccount(const ActivityAccount& other) {
	_name = other._name;
	_balance = other._balance;
	_listBoxIndex = other._listBoxIndex;
	_report = true;
}


ActivityAccount::ActivityAccount(const Activity& arg) {
	_name = arg.getName();
	_balance = arg.getTimeSpan();
	_listBoxIndex = -1;
	_report = true;
}


void ActivityAccount::addTimeSpan(const TimeSpan& span) {
	_balance += span;
}


void ActivityAccount::setTimeSpan(const TimeSpan& span) {
	_balance = span;
}


TimeSpan ActivityAccount::getTimeSpan() const {
	return _balance;
}


QString ActivityAccount::getName() const {
	return _name;
}

QString ActivityAccount::toString() const {
	QString s;
	s.sprintf("%.35s (%.5s)",_name.latin1(), _balance.toString().latin1() );
	return s;
}


bool ActivityAccount::operator<(const ActivityAccount& rhs) const {
	return this->isSmallerThan(rhs);
}


bool ActivityAccount::isSmallerThan(const ActivityAccount& other) const {
	if(this->getListBoxIndex() < other.getListBoxIndex())
		return true;
	return false;
}


bool ActivityAccount::equals(const ActivityAccount& other) const {
	if(this->_listBoxIndex == other._listBoxIndex)
		return true;
	return false;
}

bool ActivityAccount::operator==(const ActivityAccount& other) const {
	return (this->equals(other));
}


void ActivityAccount::setListBoxIndex(int idx) {
	_listBoxIndex = idx;
}


int ActivityAccount::getListBoxIndex() const {
	return _listBoxIndex;
}


void ActivityAccount::setReport(bool arg) {
    _report = arg;
}


bool ActivityAccount::getReport() const {
    return _report;
}


bool ActivityAccount::report() const {
    return getReport();
}






