// $Id: activitylist.cpp,v 1.5 2004/02/08 01:49:18 martin Exp $

// activitylist.cpp, (C) 2004 Martin Henne

#include "activitylist.h"
#include "activity.h"
#include "activityaccount.h"
#include <qdatetime.h>
#include <qsortedlist.h>
#include <qstring.h>


QSortedList<Activity> ActivityList::getPartByLastFirst(
    QSortedList<Activity> list,
    const QDate& firstDay,
    const QDate& lastDay
    ) 
{
    if(list.count() == 0) 
	return list;
    QSortedList<Activity> newList;
    Activity* a = list.first();
    while(a) {
	if( lastDay >= ( a->getStart().date() ) && 
	    firstDay <= ( a->getStart().date() ) )
	    newList.append( new Activity(*a) );
	a = list.next();
    }   
    return newList;
}


QSortedList<Activity> ActivityList::getFirstPart(
    QSortedList<Activity> list,
    const QDate& lastDay
    ) 
{
    if(list.count() == 0) 
	return list;
    QSortedList<Activity> newList;
    Activity* a = list.first();
    while(a) {
	if( lastDay >= ( a->getStart().date() ) )
	    newList.append( new Activity(*a) );
	a = list.next();
    }   
    return newList;
}


QSortedList<Activity> ActivityList::getLastPart(
    QSortedList<Activity> list,
    const QDate& firstDay
    ) 
{
    if(list.count() == 0) 
	return list;
    QSortedList<Activity> newList;
    Activity* a = list.first();
    while(a) {
	if( firstDay <= ( a->getStart().date() ) )
	    newList.append( new Activity(*a) );
	a = list.next();
    }   
    return newList;
}


QSortedList<Activity> ActivityList::getPartByName(
    QSortedList<Activity> list,
    const QString& activityName
    ) 
{
    if(list.count() == 0) 
	return list;
    QSortedList<Activity> newList;
    Activity* a = list.first();
    while(a) {
	if( activityName == a->getName() )
	    newList.append( new Activity(*a) );
	a = list.next();
    }   
    return newList;
}


QSortedList<Activity> ActivityList::exclude(
    QSortedList<Activity> list,
    const QString& nameToExclude
    )
{
    if(list.count() == 0) 
	return list;
    QSortedList<Activity> newList;
    Activity* a = list.first();
    while(a) {
	if( nameToExclude != a->getName() )
	    newList.append( new Activity(*a) );
	a = list.next();
    }
    return newList;
}


QSortedList<QString> ActivityList::getProjectNames(QSortedList<Activity> list) {
    QSortedList<QString> names;
    Activity* a = list.first();
    QString* name;
    while(a) {
	name = new QString( a->getName() );
	if( !names.contains( name ) ) {
	    names.append( name );
	}
	else {
	    delete name;
	    name = 0;
	}
	a = list.next();
    }
    return names;
}



TimeSpan ActivityList::getProjectSummary(
    QSortedList<Activity> list, 
    const QString& projectName
    ) 
{
    TimeSpan sum;
    Activity* a = list.first();
    QString name;
    while(a) {
	name = a->getName();
	if(name == projectName) {
	    sum += a->getTimeSpan();
	}
	a = list.next();
    }
    return sum;
}


TimeSpan ActivityList::getProjectSummary(
    QSortedList<Activity> list, 
    const QString& projectName,
    const QDate& day
    ) 
{
    QSortedList<Activity> dayList = ActivityList::getPartByDay(list,day);
    return ActivityList::getProjectSummary(dayList,projectName);
}


QSortedList<Activity> ActivityList::getPartByDay(
    QSortedList<Activity> list,
    const QDate& day
    ) 
{
    if(list.count() == 0) 
	return list;
    QSortedList<Activity> newList;
    Activity* a = list.first();
    while(a) {
	if( day == ( a->getStart().date() ) )
	    newList.append( new Activity(*a) );
	a = list.next();
    }   
    return newList;
}


QSortedList<ActivityAccount> ActivityList::getProjectSummaryList(
    QSortedList<Activity> list) 
{
    QSortedList<ActivityAccount> accountList;
    if(list.count()==0)
	return accountList;
    // get list of projects
    QSortedList<QString> names = ActivityList::getProjectNames(list);
    if(names.count()==0)
	return accountList;
    
    // traverse projects
    QString* projectName = names.first();
    TimeSpan span;
    while(projectName) {
	span = ActivityList::getProjectSummary(list, *projectName);
	accountList.append(new ActivityAccount(*projectName, span) );
	projectName = names.next();
    }
    
    return accountList;
}



QDate ActivityList::getFirstDay(QSortedList<Activity> list) {
    QDate d(2099,12,31);
    QDate curr;
    Activity* a = list.first();
    while(a) {
	curr = a->getStart().date();
	if(curr < d) {
	    d = curr;
	}
	a = list.next();
    }
    return d;
}


QDate ActivityList::getLastDay(QSortedList<Activity> list) {
    QDate d(1901,1,1);
    QDate curr;
    Activity* a = list.first();
    while(a) {
	curr = a->getStart().date();
	if(curr > d) {
	    d = curr;
	}
	a = list.next();
    }
    return d;    
}














