//
// C++ Implementation: composefilename
//
// Description: 
//
//
// Author: Martin Henne <martin.henne@web.de>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include "composefilename.h"
#include "composefilenamebase.h"
#include <qstring.h>
#include <cstdio>
#include <cstdlib>
#include <qlineedit.h>
#include <qdatetime.h>

ComposeFileName::ComposeFileName(const QString& defaultName)
    : ComposeFileNameBase(0,0,true,0), _default(defaultName), _rejected(false)
    {
    QString s = _default;
    lineEdit->validateAndSet(s,s.length(),0,s.length());
}

void ComposeFileName::dashSlot() {
    insert("-");
}

void ComposeFileName::dateSlot() {
    QString s;
    QDate d = QDate::currentDate();
    s.sprintf("-%.2d%.2d%.4d",d.day(),d.month(),d.year());
    QString suf = getAndRemoveSuffix();
    insert(s);
    append(suf);
}

void ComposeFileName::defaultSlot() {
    QString s = _default;
    lineEdit->validateAndSet(s,s.length(),0,s.length());
}

void ComposeFileName::dotSlot() {
    insert(".");
}

void ComposeFileName::homeSlot() {
    char* homeVar = std::getenv("HOME");
    if(!homeVar) 
	return;
    QString s;
    s.sprintf("%s/",homeVar);
    prepend(s);
}

void ComposeFileName::cardSlot() {
    prepend("/mnt/card/");
}

void ComposeFileName::cfSlot() {
    prepend("/mnt/cf/");
}    

void ComposeFileName::htmlSlot() {
    removeKnownSuffix();
    append(".html");
}

void ComposeFileName::slashSlot() {
    insert("/");
}

void ComposeFileName::timeSlot() {
    QString s;
    QTime t = QTime::currentTime();
    s.sprintf("-%.2d%.2d",t.hour(),t.minute());
    QString suf = getAndRemoveSuffix();
    insert(s);
    append(suf);
}

void ComposeFileName::removeKnownSuffix() {
    int crsr = lineEdit->cursorPosition();
    QString s = lineEdit->text();
    if(s.right(4) == ".txt") {
	s = s.left(s.length()-4);
    }
    if(s.right(4) == ".xml") {
	s = s.left(s.length()-4);
    }
    if(s.right(4) == ".dat") {
	s = s.left(s.length()-4);
    }
    if(s.right(4) == ".asc") {
	s = s.left(s.length()-4);
    }
    if(s.right(5) == ".html" ) {
	s = s.left(s.length()-5);
    }
    if(crsr > (int)s.length() )
	crsr = s.length();
    lineEdit->validateAndSet(s,crsr,0,0);
}

QString ComposeFileName::getAndRemoveSuffix() {
    QString s = lineEdit->text();
    if(s.find(".txt") >= 0) {
	removeKnownSuffix();
	return ".txt";
    }
    if(s.find(".xml") >= 0) {
	removeKnownSuffix();
	return ".xml";
    }
    if(s.find(".html") >= 0) {
	removeKnownSuffix();
	return ".html";
    }
    if(s.find(".dat") >= 0) {
	removeKnownSuffix();
	return ".dat";
    }
    if(s.find(".asc") >= 0) {
	removeKnownSuffix();
	return ".asc";
    }
    return "";
}

void ComposeFileName::txtSlot() {
    removeKnownSuffix();
    append(".txt");
}

void ComposeFileName::xmlSlot() {
    removeKnownSuffix();
    append(".xml");
}

QString ComposeFileName::show(const QString& defaultName) {
    ComposeFileName* o = new ComposeFileName(defaultName);
    o->exec();
    if(o->_rejected)
	return "";
    return o->_newName;
}

void ComposeFileName::append(const QString& arg) {
    QString s;
    s.sprintf("%s%s",lineEdit->text().latin1(), arg.latin1() );
    lineEdit->validateAndSet(s,s.length(),0,s.length());
}


void ComposeFileName::prepend(const QString& arg) {
    QString s;
    s.sprintf("%s%s", arg.latin1(), lineEdit->text().latin1() );
    lineEdit->validateAndSet(s,s.length(),0,s.length());
}


void ComposeFileName::insert(const QString& arg) {
    int cursor = lineEdit->cursorPosition();
    QString s = lineEdit->text();
    s.insert(cursor,arg);
    lineEdit->validateAndSet(s,cursor+arg.length(),0,0);
}

void ComposeFileName::accept() {
    this->_newName = lineEdit->text();
    hide();
}

void ComposeFileName::reject() {
    _rejected = true;
    this->_newName = "";
    hide();
}




