// $Id: datafile.cpp,v 1.15 2004/02/19 20:27:34 martin Exp $

// datefile.cpp, (C) 2004 Martin Henne

#include "datafile.h"
#include "io.h"
#include "activity.h"
#include "elementparser.h"
#include "mtrack.h"
#include <cstdlib>
#include <cstdio>
#include <qstring.h>
#include <qsortedlist.h>
#include <qxml.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qsortedlist.h>


DataFile::DataFile(const QString& fname) {
    if(fname == "") {
	const char* fileNameChar = std::getenv("HOME");
	std::fprintf(stderr,"DataFile::DataFile(): home is '%s'\n",fileNameChar);
	if(fileNameChar)
	    _fileName = fileNameChar;
#ifdef QWS
	_fileName += "/Applications/temtor/temtordata.xml"; // Qt/Embedded (Qtopia)
#else
	_fileName += "/.temtordata.xml"; // Qt2 (desktop)
#endif
	std::fprintf(stderr,"DataFile::DataFile(): filename is '%s'\n",_fileName.latin1());
    }
    else
	_fileName = fname;
}


DataFile::~DataFile() {
}


QSortedList<Activity> DataFile::read(QString fileName) {
    if(fileName=="")
	fileName = _fileName;
    QFile xmlFile(fileName);
    if( ! xmlFile.exists() ) {
	this->createEmty();
    }
    QSortedList<Activity> activityList;
    ElementParser* parser = new ElementParser(activityList);
    QXmlInputSource source(xmlFile);
    QXmlSimpleReader reader;
    reader.setContentHandler(parser);
    reader.parse(source);
    delete parser;
    parser = NULL;
    return activityList;
}


void DataFile::write(QSortedList<Activity> list, QString fileName) {
    list.sort();
    if(fileName=="")
	fileName = _fileName;
    QFile dataFile(fileName);
    dataFile.open(IO_WriteOnly);
    QTextStream stream(&dataFile);
    this->writeHeader(stream);
    Activity* a = list.first();
    while( a!=NULL ) {
	if(!a->isCurrent())
	    stream << a->toXmlString() << "\n";
	else
	    stream << a->toXmlString("unfinished") << "\n";
	a = list.next();
    }
    this->writeFooter(stream);
    dataFile.close();
}


void DataFile::writeHeader(QTextStream& stream) {
    stream << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
    stream << "<temtordata>\n";
    stream << "\t<temtorversion version=\"" << Mtrack::getVersion() << "\"/>\n";
    stream << "\t<temtorauthor name=\"Martin Henne\" town=\"Aachen\" email=\"martin.henne@web.de\"/>\n";
    stream << "\t<operative name=\"" << Mtrack::getOperative() << "\"/>\n";
}


void DataFile::writeFooter(QTextStream& stream) {
    stream << "</temtordata>\n";
}


void DataFile::createEmty() {
    QFile dataFile(_fileName);
    dataFile.open(IO_WriteOnly);
    QTextStream stream(&dataFile);
    this->writeHeader(stream);
    this->writeFooter(stream);
    dataFile.close();
}

void DataFile::createEmty(const QString& fileName) {
    QString oldName = _fileName;
    _fileName = fileName;
    this->createEmty();
    _fileName = oldName;
}










