// $Id: mtrack.h,v 1.22 2004/02/21 01:52:31 martin Exp $

// mtrack.h, (C) 2003 Martin Henne

/*
    This class provides general functionality that is needed by the
    application 'mTrack' and not related to a widget or another more
    suitable class.
    
    E.g. it's aim is to handle configuration and datafiles to some
    degree and to provide some global needed informations.
*/

#ifndef _mtrack_h
#define _mtrack_h

#include <qsortedlist.h>
#include "activity.h"
#include "accountlist.h"
#include "activityaccount.h"
#include <qstringlist.h>
#include <qwidget.h>

class DataFile;
class QTimer;

class Mtrack : public QObject {
  
    Q_OBJECT
    
    private:
    DataFile* _dataFile;
    
    static QSortedList<Activity> _activityList;
    static AccountList* _accountList;
    static QString _operative;
    
    void dumpActivityList(); // debug
    
    bool _tracking; // current state, tracking activity or not
    bool _timerActive;
    int _argc;
    char** _argv;
    QTimer* _saveTimer;
    
    public slots:
    void saveTimeoutSlot();
    
    public:
    static const QString _VERSION;
    Mtrack(int argc, char** argv);
    ~Mtrack();
    void mtrackStart();
    void mtrackEnd();
    AccountList* getAccountList();
    QSortedList<ActivityAccount> getProjectList();
    QStringList getProjectNamesList();
    Activity* getCurrentActivity();
    QString getCurrentActivityName();
    QSortedList<Activity>* getActivityList();
    void newCurrentActivity(const QString& name);
    void endCurrentActivity();
    void removeAccount(const QString& name);
    void renameProject(const QString& oldName, const QString& newName);
    void addActivity(Activity* arg);
    void resetActivity(const QString& name);
    void resetAll();
    void listBoxTapped(int listBoxIndex);
    QString createTimesLine();
    void save(bool now = false);
    void exportDataFile(const QString& fileName);
    void importDataFile(const QString& fileName, bool append);
    void clearAll();
    int getListBoxIndexByName(const QString& name);
    QString getNameByListBoxIndex(int index);
    bool tracking() const;
    void setTracking(bool arg);
    bool getTracking() const;
    static QString getVersion();
    Activity* getActivityByIndex(int idx);
    void removeMarked();
    void update();
    static void setOperative(const QString&);
    static QString getOperative();
    void resetToDate(const QDate& firstRemainingDay);
    void resetFromDate(const QDate& firstRemovedDay);
    void resetActivityIndex();
};

#endif
