// $Id: reporter.cpp,v 1.14 2004/02/19 20:27:34 martin Exp $

// reporter.cpp, (C) 2004 Martin Henne

#include "reporter.h"
#include "mtrack.h"
#include "activity.h"
#include "activitylist.h"
#include "timespan.h"
#include <qstring.h>
#include <qdatetime.h>
#include <qstringlist.h>

//#define DEBUG
#ifdef DEBUG
#warning debugging is active
#include <iostream>
#endif

//::header::header::header::header::header::header::header::header::header::header::header::header::header::header::header::header::header
QString Reporter::header(const QString& headLine) {
    QString s;
    s.sprintf(
	"<!-- generated temtor report //--><html>\n"
	"<head>\n<title>%s</title>\n</head>\n<body>\n",
	headLine.latin1()
	);
    return s;
}

// ::footer::footer::footer::footer::footer::footer::footer::footer::footer::footer::footer::footer::footer::footer::footer::footer::footer::footer::footer::footer::footer
QString Reporter::footer(const QString& footNote = "") {
    QString s = footNote;
    s.sprintf("</body>\n</html>\n");
    return s;
}

//::table::table::table::table::table::table::table::table::table::table::table::table::table::table::table::table::table::table::table::table::table::table::table::table
QString Reporter::table(const QString& content) {
    QString s;
    s.sprintf("<table>\n%s\n</table>",content.latin1() );
    return s;
}


QString Reporter::tableData(const QString& content, bool even) {
    QString s;
    QString evenString = "";
    if(even)
	evenString = " bgcolor=\"#a0a0ff\"";
    s.sprintf("<td%s><nobr>%s</nobr></td>",evenString.latin1(),content.latin1());
    return s;
}

//::tableRow::tableRow::tableRow::tableRow::tableRow::tableRow::tableRow::tableRow::tableRow::tableRow::tableRow::tableRow::tableRow::tableRow
QString Reporter::tableRow(
    const QString& content, 
    bool even) {
    QString s;
    QString evenString = "";
    if(even)
	evenString = " bgcolor=\"#a0a0ff\"";
    s.sprintf("  <tr%s>%s</tr>\n",evenString.latin1(),content.latin1());
    return s;   
}

// createMonthly createMonthly createMonthly createMonthly createMonthly createMonthly createMonthly createMonthly createMonthly
QString Reporter::createMonthly() {
    return "not implemented";
}

// createWeekly createWeekly createWeekly createWeekly createWeekly createWeekly createWeekly createWeekly createWeekly createWeekly
QString Reporter::createWeekly() {
    return "not implemented";
}

// createDetails  createDetails  createDetails  createDetails  createDetails  createDetails  createDetails  createDetails  createDetails  createDetails
QString Reporter::createDetails(
    const QSortedList<Activity>& list, 
    const QDate& first, 
    const QDate& last) {
    QString text;
    QString line;
    QSortedList<Activity> newList = 
	ActivityList::getPartByLastFirst(list,first,last);
    int even = 0;
    Activity* a = newList.first();
    text += Reporter::header("");
    text += "    <table>";
    while(a) {
	line = Reporter::tableData( a->toString() );
	line = Reporter::tableRow( line, ++even%2 );
	text += line;
	a = newList.next();
    }
    text += "    </table>";
    return text;
}




// createFull  createFull  createFull  createFull  createFull  createFull  createFull  createFull  createFull  createFull createFull createFull createFull
QString Reporter::createFull() {
    return "not implemented";
}




// createStandard  createStandard  createStandard  createStandard  createStandard  createStandard  createStandard  createStandard
QString Reporter::createStandard(
    QSortedList<Activity> list, 
    QStringList listOfProjects,
    const QDate& firstDay, 
    const QDate& lastDay,
    TimeSpan& carryOver,
    const float& dailyDebit,
    const int& annualLeave)
{
    QDate currentDay(firstDay);
    QString text, line, s;
    QStringList lines;
    
    // create html header
    s.sprintf("<!-- generated temtor report: %s //-->",(QDateTime::currentDateTime()).toString().latin1() );
    lines.append(s);
    lines.append("<html>");
    lines.append("  <head>");
    s.sprintf("    <title>Project Time Sheet For '%s'</title>", Mtrack::getOperative().latin1());
    lines.append(s);
    lines.append("  </head>");
    lines.append("  <body>");
    s.sprintf("    <h2>Project Time Sheet For '%s'</h2>", Mtrack::getOperative().latin1());
    lines.append(s);
    s.sprintf("    <b>First Day: </b>%s<br>", Reporter::toDateString(firstDay).latin1() );
    lines.append(s);
    s.sprintf("    <b>Last Day: </b>%s<br>", Reporter::toDateString(lastDay).latin1() );
    lines.append(s);
    if(annualLeave != 0) {
	s.sprintf("    <b>Annual Leave: </b>%i Days<br>", annualLeave );
	lines.append(s);
    }
    
    // determine no. of needed rows in each table (-> get project names)
    QSortedList<QString> projectNames;
    QStringList::Iterator iter = listOfProjects.begin(); // remove this redundance
    while(iter != listOfProjects.end() ) {
	projectNames.append( new QString(*iter) );
	std::fprintf(stderr,__FILE__ " appending '%s'\n", (*iter).latin1());
	++iter;
    }
    const int ROWS = projectNames.count();
    if(ROWS==0) 
	return "";
    
    // determine no. of needed columns over all (-> get days to report of)
    const int DAYS = firstDay.daysTo(lastDay) +1; // first day counts
    s.sprintf("    <b>Days:</b> %i<br>",DAYS);
    lines.append(s);
    s.sprintf("    <b>Projects:</b> %i<br><br>",ROWS);
    lines.append(s);
    
    // traverse day-by-day all projects
    const int PERIOD = 14;
    while(currentDay <= lastDay) {
	QDate start = currentDay;
	QDate end = currentDay.addDays(PERIOD-1);
	if(end > lastDay)
	    end = lastDay;
	s = createTableBlock(start,end,projectNames,list);
	lines.append(s);
	lines.append("    <br>");
	currentDay = currentDay.addDays(PERIOD);
    }
    
    // add summary for each project
    lines.append("      <h3>Project Summary:</h3>");
    lines.append("      <table bgcolor=\"#ffffff\">");
    QSortedList<Activity> periodList = ActivityList::getPartByLastFirst(list,firstDay,lastDay);
    iter = listOfProjects.begin();
    TimeSpan spanSum;
    while(iter != listOfProjects.end() ) {
	TimeSpan span = ActivityList::getProjectSummary(periodList, *iter);
	spanSum += span;
	s.sprintf(
	    "        <tr bgcolor=\"#c0c0c0\"><td>%s:</td><td><b><center>%s</center></b></td></tr>",
	    (*iter).latin1(), 
	    span.toShortString().latin1() );
	if(span.toShortString()!="00:00")
	    lines.append(s);
	++iter;
    }
    lines.append("        <tr bgcolor=\"#ffffff\"><td> </td><td><center>=====</center></td></tr>");
    if( !carryOver.zero() || dailyDebit != 0.0 ) {
	s.sprintf("        <tr bgcolor=\"#f0d0d0\"><td>All projects:</td><td><center>%s</center></td></tr>", spanSum.toShortString().latin1() );    
	lines.append(s);
    }
    if( !carryOver.zero() ){
	s.sprintf("        <tr bgcolor=\"#f0d0d0\"><td>Carryover:</td><td><center>%s</center></td></tr>", carryOver.toShortString().latin1() );
	lines.append(s);
	spanSum += carryOver;
    }
    if( dailyDebit!=0.0 ) {
	float debit = DAYS*dailyDebit;
	TimeSpan debitSpan(-1*debit);
	s.sprintf("        <tr bgcolor=\"#f0d0d0\"><td>Debit:</td><td><center>%s</center></td></tr>", debitSpan.toShortString().latin1() );    
	lines.append(s);
	spanSum += debitSpan;
    }
    s.sprintf("        <tr bgcolor=\"#f0d0d0\"><td>Total:</td><td><center><b>%s</b></center></td></tr>", spanSum.toShortString().latin1() );    
    lines.append(s);
    lines.append("    </table><br><br>");
    lines.append("    <i>Annotations:</i><br>");
    lines.append("    Times are measured in seconds, but shown in hours and minutes. ");
    lines.append("    Sessions are spanned over the days and shown at the day they've been started. ");
    lines.append("    This means, Sessions can sum up to more than 24 hours, but still shown at one day. ");
    lines.append("    The weekend's (Saturday and Sunday) dates are marked red.<br><br>");
    lines.append("    <hr>");
    lines.append("    <font size=\"-1\">This report is autogenerated by the time tracking ");
    lines.append("    system 'Temtor' (Tempus Relator), (C) 2004 by Martin Henne. 'Temtor' is available for ");
    lines.append("    the Sharp Zaurus and HP iPAQ. Comments: martin.henne@web.de</font>");
    lines.append("  </body>");
    lines.append("</html>");
    text = lines.join("\n");
    return text;
}


// createTableBlock  createTableBlock createTableBlock createTableBlock createTableBlock createTableBlock createTableBlock createTableBlock
QString Reporter::createTableBlock(
    const QDate& firstDay,
    const QDate& lastDay,
    QSortedList<QString>& projectNames,
    QSortedList<Activity>& list
    ) 
{
    QDate currentDay = firstDay;
    QStringList lines;
    QString s;
    // create row of days (table header)
    lines.append("    <table bgcolor=\"#ffffff\">");
    lines.append("      <tr bgcolor=\"#d0d0ff\">");
    QString red="";
    lines.append("      <td>Project\\Date</td>");
    while(currentDay <= lastDay) {
	int dow = currentDay.dayOfWeek();
	if(dow==6 || dow==7)
	    red=" bgcolor=\"#ffd0d0\"";
	else
	    red="";
	s.sprintf("        <td%s>%.2i.%.2i.</td>",red.latin1(),currentDay.day(), currentDay.month() );
	lines.append(s);
	currentDay = currentDay.addDays(1);
    }
    lines.append("      </tr>");
    
    QString *currentName = projectNames.first();
    QString spanString;
    while(currentName) {
	lines.append("      <tr>");
	s.sprintf("        <td bgcolor=\"#c0c0c0\"><nobr>%s<nobr></td>",(*currentName).latin1() );
	lines.append(s);
	currentDay = firstDay;
	while(currentDay<=lastDay) {
	    TimeSpan span = ActivityList::getProjectSummary(list,*currentName,currentDay);
	    spanString = span.toShortString();
	    if(spanString=="00:00")
		spanString = "<center>-</center>";
	    s.sprintf("        <td bgcolor=\"#f0f0f0\">%s</td>",spanString.latin1() );
	    lines.append(s);
	    currentDay = currentDay.addDays(1);
	}
	lines.append("      </tr>");
	currentName = projectNames.next();
    }
    
    lines.append("    </table>");
    return lines.join("\n");
}



QString Reporter::toDateTimeString(const QDateTime& dt) {
    QString s;
    QDate d = dt.date();
    QTime t = dt.time();
    s.sprintf(
	"%s, %d. %s %.4d, %.2d:%.2d h",
	d.dayName( d.dayOfWeek() ).latin1(),
	d.day(),
	d.monthName( d.month() ).latin1(),
	d.year(),
	t.hour(),
	t.minute());
    return s;
}



QString Reporter::toDateString(const QDate& d) {
    QString s;
    s.sprintf(
	"%s, %d. %s %.4d",
	d.dayName( d.dayOfWeek() ).latin1(),
	d.day(),
	d.monthName( d.month() ).latin1(),
	d.year());
    return s;
}
