// $Id: reportsettings.cpp,v 1.5 2004/02/11 00:30:07 martin Exp $

// reportsettings.cpp, (C) 2004 Martin Henne

#include "reportsettings.h"
#include "dateinput.h"
#include "reportsettingsbase.h"
#include "accountlist.h"
#include "timespan.h"
#include "helpview.h"
#include <qstringlist.h>
#include <qstring.h>
#include <qpushbutton.h>
#include <qlistbox.h>
#include <qspinbox.h>
#include <qradiobutton.h>
#include <qcheckbox.h>

ReportSettings::ReportSettings(
    QSortedList<ActivityAccount>& projects,
    QDate& startDate,
    QDate& endDate,
    TimeSpan& carryOver,
    float& dailyDebit,
    int& annualLeave,
    bool& accept)
    : 
    ReportSettingsBase(0,0,true,0), 
    _selectedProjects(projects), 
    _startDate(startDate), 
    _endDate(endDate),
    _carryOver(carryOver),
    _dailyDebit(dailyDebit),
    _annualLeave(annualLeave),
    _accept(accept)
    {
}



//::show()//::show()//::show()//::show()//::show()//::show()//::show()//::show()//::show()//::show()//::show()//::show()//::show()//::show()//::show()//::show()
void ReportSettings::show(
    QSortedList<ActivityAccount>& projects,
    QDate& startDate,
    QDate& endDate,
    TimeSpan& carryOver,
    float& dailyDebit,
    int& annualLeave,
    bool& accept)
{
    ReportSettings* settings = new ReportSettings(
	projects, 
	startDate, 
	endDate,
	carryOver,
	dailyDebit,
	annualLeave,
	accept);
    settings->initialize();
    settings->exec();
    delete settings;
    settings = 0;
}



//::initialize()//::initialize()//::initialize()//::initialize()//::initialize()//::initialize()//::initialize()//::initialize()//::initialize()//::initialize()//::initialize()//::initialize()
void ReportSettings::initialize() {
    std::fprintf(stderr,__FILE__ " initialize()\n");
    QString s;
    s.sprintf("End: %.2i.%.2i.%i",_endDate.day(),_endDate.month(),_endDate.year() );
    endDateButton->setText(s);
    s.sprintf("Start: %.2i.%.2i.%i",_startDate.day(),_startDate.month(),_startDate.year() );
    startDateButton->setText(s);
    
    int annualLeaveBackup = _annualLeave;
    debitSpinBox->setValue(_dailyDebit);
    _annualLeave = annualLeaveBackup;
    leaveSpinBox->setValue(_annualLeave);
    
    ActivityAccount* aa = _selectedProjects.first();
    projectListBox->clear();
    int idx = 0;
    while(aa) {
	projectListBox->insertItem( aa->getName() );
	aa->setListBoxIndex(idx);
	projectListBox->setSelected(idx,aa->getReport());
	aa = this->_selectedProjects.next();
	++idx;
    }
    debitSlot();
}



//::endDateSlot()//::endDateSlot()//::endDateSlot()//::endDateSlot()//::endDateSlot()//::endDateSlot()//::endDateSlot()//::endDateSlot()//::endDateSlot()
void ReportSettings::endDateSlot() {
    std::fprintf(stderr,__FILE__ " endDateSlot()\n");
    QString s;
    this->_endDate = DateInput()(_endDate);
    s.sprintf("End: %.2i.%.2i.%i",_endDate.day(),_endDate.month(),_endDate.year() );
    endDateButton->setText(s);
}


//::startDateSlot()//::startDateSlot()//::startDateSlot()//::startDateSlot()//::startDateSlot()//::startDateSlot()//::startDateSlot()//::startDateSlot()
void ReportSettings::startDateSlot() {
    std::fprintf(stderr,__FILE__ " startDateSlot()\n");
    QString s;
    this->_startDate = DateInput()(_startDate);
    s.sprintf("Start: %.2i.%.2i.%i",_startDate.day(),_startDate.month(),_startDate.year() );
    startDateButton->setText(s);
}


//::projectSlot()//::projectSlot()//::projectSlot()//::projectSlot()//::projectSlot()//::projectSlot()//::projectSlot()//::projectSlot()//::projectSlot()//::projectSlot()
void ReportSettings::projectSlot() {
    int currentIndex = projectListBox->currentItem();
    bool selected = projectListBox->isSelected( currentIndex );
    
    ActivityAccount* aa = _selectedProjects.first();
    while(aa) {
	if(aa->getListBoxIndex() == currentIndex) {
	    aa->setReport(selected);
	    std::fprintf(stderr,__FILE__" projectSlot(): changed status of '%s' to %d\n",aa->getName().latin1(),(int)(aa->report()));
	    break;
	}
	aa = this->_selectedProjects.next();
    }
}


//::quickReportSlot()//::quickReportSlot()//::quickReportSlot()//::quickReportSlot()//::quickReportSlot()//::quickReportSlot()//::quickReportSlot()
void ReportSettings::quickReportSlot() {
    // TODO remove me
}


//::selectAllSlot()//::selectAllSlot()//::selectAllSlot()//::selectAllSlot()//::selectAllSlot()//::selectAllSlot()//::selectAllSlot()//::selectAllSlot()
void ReportSettings::selectAllSlot() {
    ActivityAccount* aa = _selectedProjects.first();
    int idx = 0;
    while(aa) {
	aa->setReport(true);
	projectListBox->setSelected(idx,aa->getReport());
	aa = this->_selectedProjects.next();
	++idx;
    }
}


//::selectNoneSlot()//::selectNoneSlot()//::selectNoneSlot()//::selectNoneSlot()//::selectNoneSlot()//::selectNoneSlot()//::selectNoneSlot()
void ReportSettings::selectNoneSlot() {
    ActivityAccount* aa = _selectedProjects.first();
    int idx = 0;
    while(aa) {
	aa->setReport(false);
	projectListBox->setSelected(idx,aa->getReport());
	aa = this->_selectedProjects.next();
	++idx;
    }
}


//::carryOverSlot()//::carryOverSlot()//::carryOverSlot()//::carryOverSlot()//::carryOverSlot()//::carryOverSlot()//::carryOverSlot()//::carryOverSlot()
void ReportSettings::carryOverSlot() {
    std::fprintf(stderr,__FILE__ " carryOverSlot()\n");
    int hour = hourSpinBox->value();
    int minute = minuteSpinBox->value();
    if(minusButton->isChecked() ) {
	hour *= -1;
	minute *= -1;
    }
    this->_carryOver = TimeSpan(hour,minute,0);
}



//::minusSlot()//::minusSlot()//::minusSlot()//::minusSlot()//::minusSlot()//::minusSlot()//::minusSlot()//::minusSlot()//::minusSlot()//::minusSlot()
void ReportSettings::minusSlot() {
    int hour = _carryOver.hours();
    if(hour>0 && minusButton->isChecked() ) {
	_carryOver.hours( hour*-1 );
	_carryOver.minutes( _carryOver.minutes()*-1 );
	_carryOver.seconds( _carryOver.seconds()*-1 );
    }
    if(hour<0 && ( !minusButton->isChecked() ) ) {
	_carryOver.hours( hour*-1 );
	_carryOver.minutes( _carryOver.minutes()*-1 );
	_carryOver.seconds( _carryOver.seconds()*-1 );
    }
}



//::debitSlot()//::debitSlot()//::debitSlot()//::debitSlot()//::debitSlot()//::debitSlot()//::debitSlot()//::debitSlot()//::debitSlot()//::debitSlot()
void ReportSettings::debitSlot() {
}


//::dailySlot()//::dailySlot()//::dailySlot()//::dailySlot()//::dailySlot()//::dailySlot()//::dailySlot()//::dailySlot()//::dailySlot()//::dailySlot()
void ReportSettings::dailySlot() {
    if(debitSpinBox->value() > 24)
	debitSpinBox->setValue(24);
    this->setDailyDebit( debitSpinBox->value() );
}


//::weeklySlot()//::weeklySlot()//::weeklySlot()//::weeklySlot()//::weeklySlot()//::weeklySlot()//::weeklySlot()//::weeklySlot()//::weeklySlot()
void ReportSettings::weeklySlot() {
    if(debitSpinBox->value() > (24*7))
	debitSpinBox->setValue(24*7);    
    int value = debitSpinBox->value();
    this->setDailyDebit( ( 1.0 * value ) / ( ( 365.25 - _annualLeave ) / ( 365.25 / 7.0 ) ) );
}


//::monthlySlot()//::monthlySlot()//::monthlySlot()//::monthlySlot()//::monthlySlot()//::monthlySlot()//::monthlySlot()//::monthlySlot()
void ReportSettings::monthlySlot() {
    if(debitSpinBox->value() > (24*31))
	debitSpinBox->setValue(24*31);
    int value = debitSpinBox->value();
    this->setDailyDebit( ( 1.0 * value ) / ( ( 365.25 - _annualLeave ) / 12.0 ) );
}


//::annualSlot()//::annualSlot()//::annualSlot()//::annualSlot()//::annualSlot()//::annualSlot()//::annualSlot()//::annualSlot()//::annualSlot()
void ReportSettings::annualSlot() {
    if(debitSpinBox->value() > (24*366))
	debitSpinBox->setValue(24*366);
    int value = debitSpinBox->value();
    this->setDailyDebit( ( 1.0 * value ) / ( 365.25 - _annualLeave ) );
    
}


//::leaveSlot()//::leaveSlot()//::leaveSlot()//::leaveSlot()//::leaveSlot()//::leaveSlot()//::leaveSlot()//::leaveSlot()//::leaveSlot()//::leaveSlot()
void ReportSettings::leaveSlot() {
}


//::setDailyDebit()//::setDailyDebit()//::setDailyDebit()//::setDailyDebit()//::setDailyDebit()//::setDailyDebit()//::setDailyDebit()//::setDailyDebit()
void ReportSettings::setDailyDebit(float arg) {
    if(arg>24)
	arg=24;
    _dailyDebit = arg;
    std::fprintf(stderr,__FILE__" _dailyDebit = '%f'\n",_dailyDebit);
}


//::dailyDebitSlot()//::dailyDebitSlot()//::dailyDebitSlot()//::dailyDebitSlot()//::dailyDebitSlot()//::dailyDebitSlot()//::dailyDebitSlot()
void ReportSettings::dailyDebitSlot() {
    this->_annualLeave = leaveSpinBox->value();    
    if( dailyButton->isChecked() ) {
	dailySlot();
	return;
    }
    if( weeklyButton->isChecked() ) {
	weeklySlot();
	return;
    }
    if( monthlyButton->isChecked() ) {
	monthlySlot();
	return;
    }
    if( annualButton->isChecked() ) {
	annualSlot();
	return;
    }
}



void ReportSettings::reject() {
    _accept = false;
    hide();
}

void ReportSettings::accept() {
    _accept = true;
    hide();
}


void ReportSettings::helpReportSettingsSlot() {
    helpReportSettings();
}


void ReportSettings::helpReportSettings() {
    QString s;
    s =
	"<b>Report Settings Help</b><br>"
	"This document describes the elements of the Report Settings Dialog: "
	"<br><br>"
	"<b>- Start/End Buttons</b><br>"
	"Those buttons are provided to set the period for which the report will be "
	"generated.<br><br>"
	"<b>- Choose Projects</b><br>"
	"Only those projects, which are selected in this box, are included in the "
	"report. All calculations are based only on those projects. The select "
	"all/none buttons accelerate your selection...<br><br>"
	"<b>- Carryover</b><br>"
	"In case you want some time to be carried over from a past period, you can "
	"set it here in hours and minutes. Check the 'minus' box, if the time is negative. "
	"If you enter 0 here, no carryover is mentioned at the report."
	"<br><br>"
	"<b>- Debit</b><br>"
	"Insert the time you have to work on a specific project by a contract (e.g. "
	"a contract of employment). This time will be subtracted at the end of the "
	"contract at the summary section. If no debit is set (value 0), the debit is "
	"not even mentioned at the report.<br><br>"
	"<b>- Annual Leave</b><br>"
	"If you abide by a contract, this contract will probably contain "
	"an arrangement for your holidays. Enter the number of the specified "
	"days of your annual leave here.<br><br>"
	""
	"";
    HelpView* hv = new HelpView(s);
    hv->exec();
    delete hv;
    hv = 0;
}


