// $Id: window.cpp,v 1.39 2004/02/27 02:08:12 martin Exp $

// window.cpp, (C) 2003 Martin Henne

#include "windowbase.h"
#include "datepicker.h"
#include "composefilename.h"
#include "window.h"
#include "inputdialog.h"
#include "quickdate.h"
#include "reporter.h"
#include <qtextstream.h>
#include <qstring.h>
#include <qtabwidget.h>
#include <qlistbox.h>
#include <qcombobox.h>
#include <qthread.h>
#include <qwidget.h>
#include <qtextview.h>
#include <qlabel.h>
#include <qsortedlist.h>
#include <qfile.h>
#ifdef QWS
#include <qpe/fileselector.h>
#include <qpe/datebookmonth.h>
#endif
#include "io.h"
#include "usertextinput.h"
#include "datetimeinput.h"
#include "mtrack.h"
#include "timespan.h"
#include "activitydetails.h"
#include "activitylist.h"
#include "dateinput.h"
#include "activityaccount.h"
#include "projecthelp.h"
#include "modifyhelp.h"
#include "reportsettings.h"
#include "renamedialog.h"

Window::Window(
    Mtrack* mtrack,
    QString& userInput,
    QWidget* parent,
    const char* name,
    WFlags flags
    )
    :
    WindowBase(parent,name,flags),
    _userInput(userInput),
    _mtrack(mtrack),
    _timer(0),
    _updates(0),
    _modifyListCreated(false),
    _carryOver(0.0f),
    _dailyDebit(0.0),
    _annualLeave(0)
    {
    listBox->setSelectionMode(QListBox::Single);
    _timer = new QTimer(this);
    connect (_timer, SIGNAL( timeout() ), this, SLOT( updateTimesSlot() ) );
    modifyListBox->clear();
    modifyListBox->insertItem("Select a project.");
    // modifyListBox->setTextFormat(RichText); // Schade, gibs nich!
    reportTextView->setTextFormat(RichText);
    
    //connect( tabWidget, SIGNAL( currentChanged(QWidget*) ), this, SLOT( tabChangedSlot() ) );
}



//::tabChangedSlot()//::tabChangedSlot()//::tabChangedSlot()//::tabChangedSlot()//::tabChangedSlot()//::tabChangedSlot()
void Window::tabChangedSlot() {
    //std::fprintf(stderr,"Window::tabChangedSlot(): changed to '%i'\n",tabWidget->currentPageIndex() );
    modifyComboBox->clear();
    modifyListBox->clear();
    modifyListBox->insertItem("Select a project.");
    this->_modifyListCreated = false;
    int currentTab = tabWidget->currentPageIndex();
    if(currentTab==1) {
	QSortedList<ActivityAccount> accounts = _mtrack->getProjectList();
	ActivityAccount* account = accounts.first();
	int idx = -1;
	while(account) {
	    modifyComboBox->insertItem(account->getName());
	    account->setListBoxIndex(++idx);
	    account = accounts.next();
	}
	//this->generateActivityListSlot();
    }
}


Window::~Window() {
    delete _timer;
    _timer = 0;
}



//::comboBoxSlot()//::comboBoxSlot()//::comboBoxSlot()//::comboBoxSlot()//::comboBoxSlot()//::comboBoxSlot()//::comboBoxSlot()//::comboBoxSlot()
void Window::comboBoxSlot() {
    QString currentText = comboBox->currentText();
    // Test: DateTimeDialog
    if(currentText == "(DateTimeDialog)") {
	QDateTime dt = DateTimeInput()();
	println()( QString("-> ")+dt.toString() );
	return;
    }
    // New Activity
    if(currentText == "New Project")
	return newActivitySlot();
    // clear
    if(currentText == "Remove All Projects") {
	return clearAllSlot();
    }
    
    // remove current selected
    if(currentText == "Remove Project") {
	return removeActivitySlot();
    }
    
    // reset activity
    if(currentText == "Reset Project") {
	return resetActivitySlot();
    }
    // reset all
    if(currentText == "Reset All Projects") {
	if(!askYesNo()("really reset all accounts?"))
	    return;
	return resetAllSlot();
    }
    
    // rename project
    if(currentText == "Rename Project...") {
	QStringList projectNames = _mtrack->getProjectNamesList();
	QString oldName, newName;
	RenameDialog* rd = new RenameDialog(
	    projectNames,
	    oldName,
	    newName);
	rd->exec();
	std::fprintf(stderr,"renaming '%s' to '%s'\n",oldName.latin1(), newName.latin1());
	delete rd; rd=0;
	_mtrack->renameProject(oldName,newName);
	Activity* currentActivity = _mtrack->getCurrentActivity();
	if(!currentActivity)
	    _mtrack->setTracking(false);
	this->showAccounts();
	_mtrack->save();
	return;
    }
    
    // remove from date
    if(currentText == "Reset All After Date...") {
	
	QDate d = DateInput()();
	
	if(askYesNo()("Really Reset?")) {
	    _mtrack->resetFromDate(d);
	    update();
	}
	return;
    }
    
    
    // remove from date
    if(currentText == "Reset All Before Date...") {
	QDate d = DateInput()();
	if(askYesNo()("Really Reset?")) {
	    _mtrack->resetToDate(d);
	    update();
	}
	return;
    }
    
    if(currentText == "Export Datafile...") {
	QString newFileName = ComposeFileName::show("temtordata.xml");
	if(newFileName=="") {
	    std::fprintf(stderr,__FILE__" export canceled\n");
	    return;
	}
	_mtrack->exportDataFile(newFileName);
	return;
    }
    
    if(currentText == "Import Datafile...") {
	QString newFileName = ComposeFileName::show("temtordata.xml");
	if(newFileName=="") {
	    std::fprintf(stderr,__FILE__" import canceled\n");
	    return;
	}
	_mtrack->importDataFile(newFileName,false); // TODO implement append function
	this->update();
	return;
    }
    
    // Fallback
    alert()("Not yet implemented");
    //println()(QString(__FILE__)+" ComboBox");
}


//::reportComboBoxSlot()//::reportComboBoxSlot()//::reportComboBoxSlot()//::reportComboBoxSlot()//::reportComboBoxSlot()//::reportComboBoxSlot()
void Window::reportComboBoxSlot() {
    QString currentText = reportComboBox->currentText();
    if(currentText == "Set Operative...") {
	QString s = UserTextInput()("Set Operative","Enter Name!");
	Mtrack::setOperative(s);
    }
    if(currentText == "Create...") {
	QSortedList<ActivityAccount> accounts = _mtrack->getProjectList();
	QSortedList<Activity>* activities = _mtrack->getActivityList();
	QDate start = ActivityList::getFirstDay(*activities);
	QDate end = ActivityList::getLastDay(*activities);
	
	bool accept = false;
	std::fprintf(stderr,"Window::reportComboBoxSlot(): _annualLeave(%i)\n",_annualLeave);
	ReportSettings::show(accounts,start,end,_carryOver,_dailyDebit,_annualLeave,accept);
	if(accept)
	    this->reportStandard(accounts,start,end,_carryOver,_dailyDebit,_annualLeave);
    }
    if(currentText == "Save Report...") {
	saveReport();
    }
    if(currentText == "(test)") {
	test();
    }
}


//::newActivitySlot()//::newActivitySlot()//::newActivitySlot()//::newActivitySlot()//::newActivitySlot()//::newActivitySlot()//::newActivitySlot()
void Window::newActivitySlot() {
    QString text = UserTextInput()("New Activity","Enter description!");
    if(text.length() != 0) {
	Activity* a = new Activity(text);
	_mtrack->addActivity(a);
	showAccounts();
    }
    else
	println()("text input canceled");
    
    _mtrack->save();
}



void Window::createAndModifyActivitySlot() {
    createAndModifyActivity();
}



//::createAndModifyActivity()//::createAndModifyActivity()//::createAndModifyActivity()//::createAndModifyActivity()//::createAndModifyActivity()
void Window::createAndModifyActivity() {
    QString name = UserTextInput()("Create Activity","Enter Projectname");
    if(name=="")
	return;
    Activity* a = new Activity(name);
    ActivityDetails()(a,false);
    if(a->getName()=="") {
	alert()("Canceled!");
	return;
    }
    _mtrack->addActivity(a);
    showAccounts();
    this->generateActivityListSlot(a->getName());
}


//::clearAllSlot()//::clearAllSlot()//::clearAllSlot()//::clearAllSlot()//::clearAllSlot()//::clearAllSlot()//::clearAllSlot()//::clearAllSlot()
void Window::clearAllSlot() {
    // TODO ask before really deleting all activities
    _mtrack->clearAll();
    listBox->clear();
    _mtrack->save();
}


//::showActivityAccount()//::showActivityAccount()//::showActivityAccount()//::showActivityAccount()//::showActivityAccount()
void Window::showActivityAccount(const QString& name, const TimeSpan& time) {
    QString s;
    s.sprintf( "%.35s (%.5s)", name.latin1(), time.toShortString().latin1() );
    listBox->insertItem(s);
}



//::showAccounts()//::showAccounts()//::showAccounts()//::showAccounts()//::showAccounts()//::showAccounts()//::showAccounts()
void Window::showAccounts() {
    std::fprintf(stderr,"Window::showAccounts() called\n");
    listBox->clear();
    AccountList* aList = _mtrack->getAccountList();
    int i=0; // listboxindex
    if(aList->empty()) {
	updateTimesLine();
	return;
    }
    //bool nextAvailable = aList->hasNext();
    while( aList->hasNext() ) {
	ActivityAccount* current = aList->getNext();
	if(!current) {
	    break;
	}
	aList->setListBoxIndex(current->getName(), i++);
	QString s;
	QString trackedActivityName = _mtrack->getCurrentActivityName();
	QString currentProcessedActivityName = current->getName();
	if( trackedActivityName != currentProcessedActivityName )
	    s.sprintf("%.1d. %s", i, current->toString().latin1());
	else
	    s.sprintf("%.1d. %s (*)", i, current->getName().latin1() );
	listBox->insertItem( s );
    }
    Activity* currentActivity = _mtrack->getCurrentActivity();
    if(!currentActivity) {
	updateTimesLine();
	return;
    }
    if(!currentActivity->empty()) {
	QString selectedName = currentActivity->getName();
	setSelected( selectedName );
    }
}



//::removeActivitySlot()//::removeActivitySlot()//::removeActivitySlot()//::removeActivitySlot()//::removeActivitySlot()//::removeActivitySlot()
void Window::removeActivitySlot() {
    QString current = _mtrack->getCurrentActivityName();
    QString name = UserTextInput()("Remove Project","Remove which project?",current);
    QString s;
    s.sprintf("Remove '%s'",name.latin1());
    if(!askYesNo()(s)) {
	return;
    }
    if(current == name)
	_mtrack->endCurrentActivity();
    _mtrack->removeAccount(name);
    update();
}



//::update()//::update()//::update()//::update()//::update()//::update()//::update()//::update()//::update()//::update()//::update()//::update()//::update()
void Window::update() {
    Activity* currentActivity = _mtrack->getCurrentActivity();
    if(!currentActivity)
	_mtrack->setTracking(false);
    _mtrack->update();
    this->showAccounts();
    _mtrack->save();
}



//::initialize()//::initialize()//::initialize()//::initialize()//::initialize()//::initialize()//::initialize()//::initialize()//::initialize()//::initialize()//::initialize()
void Window::initialize() {
    if( _mtrack->tracking() ) {
	_timer->start(1000);
    }
}



//::tapOnListSlot()::tapOnListSlot()::tapOnListSlot()::tapOnListSlot()::tapOnListSlot()::tapOnListSlot()::tapOnListSlot()::tapOnListSlot()::tapOnListSlot()
void Window::tapOnListSlot() {
    std::fprintf(stderr,"Window::tapOnListSlot(): called\n");
    int currentIndex = listBox->currentItem();
    std::fprintf(stderr,"Window::tapOnListSlot(): current index '%d'\n",currentIndex);
    if(currentIndex<0) return;
    _mtrack->listBoxTapped(currentIndex);
    std::fprintf(stderr,"Window::tapOnListSlot(): back from Mtrack::listBoxTapped()\n");
    if( _timer->isActive() ) {
	_timer->stop();
    }
    _timer->start(1000);
    std::fprintf(stderr,"Window::tapOnListSlot(): _timer started\n");
    QString timesLine = _mtrack->createTimesLine();
    activityBalance->setText(timesLine);
    showAccounts();
    listBox->setSelected(currentIndex,true);
    _mtrack->save();
}


// for autosave and gui updates
//::updateTimesSlot()::updateTimesSlot()::updateTimesSlot()::updateTimesSlot()::updateTimesSlot()::updateTimesSlot()::updateTimesSlot()
void Window::updateTimesSlot() {
    if(!_mtrack->tracking()) return;
    if(!_timer)
	_timer = new QTimer(this);
    _timer->start(1000);
    ++_updates;
    if(_updates%300 == 0) { // 5 minutes is enough for autosave
	_mtrack->save();
    }
    updateTimesLine(); // update the times 
}



//::setTimesLine()::setTimesLine()::setTimesLine()::setTimesLine()::setTimesLine()::setTimesLine()::setTimesLine()::setTimesLine()::setTimesLine()
void Window::setTimesLine(const QString& line) {
    activityBalance->setText(line);
}



//::updateTimesLine()::updateTimesLine()::updateTimesLine()::updateTimesLine()::updateTimesLine()::updateTimesLine()::updateTimesLine()
void Window::updateTimesLine() {
    this->setTimesLine( _mtrack->createTimesLine() );
}



//::setSelected()::setSelected()::setSelected()::setSelected()::setSelected()::setSelected()::setSelected()::setSelected()::setSelected()
void Window::setSelected(const QString& name) {
    int index = _mtrack->getListBoxIndexByName(name);
    if(index==-1)
	return;
    listBox->setSelected(index,true);
}



//::resetActivitySlot()::resetActivitySlot()::resetActivitySlot()::resetActivitySlot()::resetActivitySlot()::resetActivitySlot()::resetActivitySlot()
void Window::resetActivitySlot() {
    QString current = _mtrack->getCurrentActivityName();
    QString name = UserTextInput()("Reset Project","Reset which project?",current);
    QString s;
    s.sprintf("Reset '%s'",name.latin1());
    if(!askYesNo()(s)) {
	return;
    }
    if(current == name)
	_mtrack->endCurrentActivity();
    _mtrack->resetActivity(name);
    update();
}



//::resetAllSlot()::resetAllSlot()::resetAllSlot()::resetAllSlot()::resetAllSlot()::resetAllSlot()::resetAllSlot()::resetAllSlot()::resetAllSlot()::resetAllSlot()
void Window::resetAllSlot() {
    _mtrack->resetAll();
    _mtrack->save();
    showAccounts();
}



//::setVersion()::setVersion()::setVersion()::setVersion()::setVersion()::setVersion()::setVersion()::setVersion()::setVersion()::setVersion()
void Window::setVersion(const QString& version) {
    QString s;
    s.sprintf("%s (%s, %s)", version.latin1(), __DATE__,__TIME__);
    versionLabel->setText(s);
}



//::tapOnModifyListSlot()::tapOnModifyListSlot()::tapOnModifyListSlot()::tapOnModifyListSlot()::tapOnModifyListSlot()::tapOnModifyListSlot()
void Window::tapOnModifyListSlot() {
    std::fprintf(stderr,"Window::tapOnModifyListSlot() called\n");
    int modifyComboBoxIndex = modifyComboBox->currentItem();
    QString selectedProject = modifyComboBox->text(modifyComboBoxIndex);
    if( _mtrack->tracking() )
	if(!askYesNo()("This stops current activity, continue?")) {
	    return;
	}
    _mtrack->endCurrentActivity();
    if(!this->_modifyListCreated) {
	modifyListBox->insertItem("Please choose a project first!");
	return;
    }
    int index = this->modifyListBox->currentItem();
    if(index < 0) {
	modifyListBox->clear();
	modifyListBox->insertItem("Please choose a project above!");
	return;
    }
    Activity* a = this->_mtrack->getActivityByIndex(index);
    if(!a) 
	return;
    ActivityDetails()(a);
    if(a->getName() == "--REMOVEME--") {
	_mtrack->removeMarked();
    }
    modifyListBox->clear();
    this->generateActivityListSlot(selectedProject);
    _mtrack->update();
    showAccounts();
}



//::generateActivityListSlot()::generateActivityListSlot()::generateActivityListSlot()::generateActivityListSlot()::generateActivityListSlot()
void Window::generateActivityListSlot(const QString& projectName) {
    std::fprintf(stderr,"Window::generateActivityListSlot()\n");
    modifyListBox->clear();
    _mtrack->resetActivityIndex();
    QSortedList<Activity>* list = new QSortedList<Activity>;
    list->setAutoDelete(false);
    QSortedList<Activity>* fullList = _mtrack->getActivityList();
    Activity* a = fullList->first();
    QString s;
    QDate d;
    while(a) {
	if(a->getName() == projectName)
	    list->append(a);
	a = fullList->next();
    }
    
    int index = -1;
    a = list->first();
    if(!a) {
	this->_modifyListCreated = false;
	this->modifyListBox->insertItem("(no activities so far)");
	return;
    }
    while(a) {
	d = a->getStart().date();
	s.sprintf(
	    "%d. Time: %s, Start: %s, %d. %s. %.4d",
	    index+2, 
	    a->getDuration().toShortString().latin1(),
	    d.dayName(d.dayOfWeek() ).latin1(),
	    d.day(),
	    d.monthName(d.month()).latin1(),
	    d.year());
	modifyListBox->insertItem( s.latin1() );
	a->setIndex(++index);
	a = list->next();
    }
    delete list;
    list = 0;
    this->_modifyListCreated = true;
}



//::modifyComboBoxSlot()//::modifyComboBoxSlot()//::modifyComboBoxSlot()//::modifyComboBoxSlot()//::modifyComboBoxSlot()
void Window::modifyComboBoxSlot() {
    std::fprintf(stderr,"Window::modifyComboBoxSlot()\n");
    int idx = modifyComboBox->currentItem();
    QString name = modifyComboBox->text(idx);
    this->generateActivityListSlot(name);
}



//::projectHelpSlot()::projectHelpSlot()::projectHelpSlot()::projectHelpSlot()::projectHelpSlot()::projectHelpSlot()::projectHelpSlot()::projectHelpSlot()
void Window::projectHelpSlot() {
    ProjectHelp()();
}



//::modifyHelpSlot()::modifyHelpSlot()::modifyHelpSlot()::modifyHelpSlot()::modifyHelpSlot()::modifyHelpSlot()::modifyHelpSlot()::modifyHelpSlot()
void Window::modifyHelpSlot() {
    ModifyHelp()();
}



//::lastMonthSlot()::lastMonthSlot()::lastMonthSlot()::lastMonthSlot()::lastMonthSlot()::lastMonthSlot()::lastMonthSlot()::lastMonthSlot()::lastMonthSlot()
void Window::lastMonthSlot() {
    // TODO remove me
}



//::reportDetails()::reportDetails()::reportDetails()::reportDetails()::reportDetails()::reportDetails()::reportDetails()::reportDetails()::reportDetails()
void Window::reportDetails() {
    QSortedList<Activity> list = *( _mtrack->getActivityList() );
    QDate start(1,1,1);
    QDate end = QDate::currentDate();
    QString s = Reporter::createDetails(list,start,end);
    std::fprintf(stderr,s.latin1());
    this->reportTextView->append(s);
}


//::reportFull()::reportFull()::reportFull()::reportFull()::reportFull()::reportFull()::reportFull()::reportFull()::reportFull()::reportFull()::reportFull()::reportFull()
void Window::reportFull() {
    // TODO remove me
}



//::reportStandard::reportStandard::reportStandard::reportStandard::reportStandard::reportStandard::reportStandard::reportStandard::reportStandard
void Window::reportStandard(
    QSortedList<ActivityAccount> accounts,
    const QDate& start, 
    const QDate& end, 
    TimeSpan& carryOver,
    const float& dailyDebit,
    const int& annualLeave)
{
    QSortedList<Activity> list = *( _mtrack->getActivityList() );
    //QSortedList<ActivityAccount> accounts = _mtrack->getProjectList();
    QStringList projects;
    ActivityAccount* account = accounts.first();
    while(account) {
	if(account->report())
	    projects.append( account->getName() );
	account = accounts.next();
    }
    QString s = Reporter::createStandard(list, projects, start, end, carryOver, dailyDebit, annualLeave );
    std::printf("%s\n",s.latin1());
    this->reportTextView->setText(s);
    //this->reportTextView->append(s);
}


//::saveReport()//::saveReport()//::saveReport()//::saveReport()//::saveReport()//::saveReport()//::saveReport()//::saveReport()//::saveReport()
void Window::saveReport() {
    QString fileName = ComposeFileName::show("temtorreport.html");
    if(fileName=="") {
	std::fprintf(stderr,__FILE__ " saveReport() canceled\n");
	return;
    }
    // now save the beast
    QFile file(fileName);
    if( !file.open(IO_WriteOnly) ) {
	alert()("Could not write to file!");
	return;
    }
    QTextStream stream(&file);
    stream << reportTextView->text();
    file.close();
}



//::helpReportSlot()//::helpReportSlot()//::helpReportSlot()//::helpReportSlot()//::helpReportSlot()//::helpReportSlot()//::helpReportSlot()
void Window::helpReportSlot() {
    this->helpReport();
}


//::helpReport()//::helpReport()//::helpReport()//::helpReport()//::helpReport()//::helpReport()//::helpReport()//::helpReport()//::helpReport()
void Window::helpReport() {
    QString s = 
	"<h2>\n"
	"What You Can Do here: \n"
	"</h2>\n"
	"1. Tab 'Create...' at the combo box<br> below to create a html report. \n"
	"<br><br>\n"
	"2. Choose 'Save Report...' to save the <br>content of this window to \n"
	"a html file.\n"
	"<br><br>\n"
	"3. Set your (the operative's) name, that <br>is shown on the report. \n";
    reportTextView->setText(s);
}



void Window::test() {
    QDate d = QDate::currentDate();
    d = DatePicker::show(QPoint(10,50));
    std::fprintf(stderr,__FILE__ " test(): %s\n", d.toString().latin1() );
}





